<?php declare(strict_types=1);

namespace EuKit\Base\Parts;

use Exception;
use Infomaker\Imengine\Imengine;
use Infomaker\Everyware\Twig\View;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\Storage\Cache;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Base\Interfaces\NewsItemPresentation;

class ArticleBodyPresentation implements NewsItemPresentation
{
    private $relatedArticles;

    public function __construct(PropertyObject $article)
    {
        $this->relatedArticles = [];

        foreach ($article->get('articles', []) as $relatedArticle) {
            $this->relatedArticles[$relatedArticle->uuid] = (object)[];
            if ($relatedArticle->permalink !== null) {
                $this->relatedArticles[$relatedArticle->uuid]->permalink = $relatedArticle->permalink;
            }
            if ($relatedArticle->teaser_image_uuids !== null && ! empty($relatedArticle->teaser_image_uuids)) {
                $this->relatedArticles[$relatedArticle->uuid]->thumbnail = $relatedArticle->teaser_image_uuids[0];
            }
            if ($relatedArticle->section !== null) {
                $this->relatedArticles[$relatedArticle->uuid]->section = $relatedArticle->section;
            }
        }
    }

    protected function element(Item $item): string
    {
        $content = $this->isListElement($item) ? $this->generateListContent($item) : $item->get('content');

        return $this->generateArticlePart('element', [
            'type' => str_replace('x-im/', '', $item->getType()),
            'content' => $content ? htmlspecialchars_decode($content) : '',
        ]);
    }

    private function image(ImageItem $item): string
    {
        $image = new Image($item);

        return $this->generateArticlePart('image', $image->getViewData());
    }

    private function table(Item $item): string
    {
        return $this->generateArticlePart('table', $item->toArray());
    }

    private function htmlEmbed(Item $item): string
    {
        return $this->generateArticlePart('html-embed', $item->toArray());
    }

    private function imageGallery(Item $item): string
    {
        $images = array_map([&$this, 'image'], (array)$item->get('images'));

        $item->set('images', $images);

        return $this->generateArticlePart('gallery', $item->toArray());
    }

    private function contentPart(Item $item): string
    {
        $contentPartText = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $contentPartText));

        return $this->generateArticlePart('content-part', $item->toArray());
    }

    private function relatedArticle(Item $item): string
    {
        if ( ! isset($item->uuid)) {
            return '';
        }

        if (isset($this->relatedArticles[$item->uuid])) {
            $item->set('permalink', $this->relatedArticles[$item->uuid]->permalink);
            $item->set('section', $this->relatedArticles[$item->uuid]->section);

            if (isset($this->relatedArticles[$item->uuid]->thumbnail)) {
                $thumbnail = Imengine::create()->fit(100)->fromUuid($this->relatedArticles[$item->uuid]->thumbnail);
                $item->set('thumbnail', $thumbnail);
            }

            return $this->generateArticlePart('related-article', $item->toArray());
        }

        return '';
    }

    private function pdf(Item $item): string
    {
        return $this->generateArticlePart('pdf', [
            'url' => $item->get('url'),
            'title' => $item->get('title'),
        ]);
    }

    private function youtube(Item $item): string
    {
        return $this->generateArticlePart('youtube', $item->toArray());
    }

    private function review(Item $item): string
    {
        $text = array_map([&$this, 'element'], (array)$item->get('text'));

        $item->set('text', implode('', $text));

        return $this->generateArticlePart('review', $item->toArray());
    }

    private function youplay(Item $item): string
    {
        return $this->generateArticlePart('youplay', $item->toArray());
    }

    private function socialEmbed(Item $item): ?string
    {
        $links = $item->get('content');

        if (empty($links)) {
            return '';
        }

        if ( ! ($linkItem = array_shift($links)) instanceof Item) {
            return '';
        }
        switch ($linkItem->getType()) {
            case 'x-im/tweet':
                return $this->twitter($linkItem);
                break;
            case 'x-im/facebook-post':
                return $this->facebook($linkItem);
                break;
            case 'x-im/instagram':
                return $this->instagram($linkItem);
                break;
            case 'x-im/vimeo':
                return $this->vimeo($linkItem);
                break;
            default:
                return '';
                break;
        }
    }

    private function instagram(Item $item): string
    {
        $embed_url = $item->embed_url ?? $item->url ?? '';
        $key = md5("instagram_{$embed_url}");

        if (($content = Cache::get($key)) === false) {
            try {
                if (isset($item->url)) {
                    $content = file_get_contents('https://api.instagram.com/oembed/?url=' . $item->url);
                }
            } catch (Exception $e) {
                return '';
            }

            Cache::set($key, $content, 5 * MINUTE_IN_SECONDS);
        }

        return $this->generateArticlePart('instagram', [
            'content' => json_decode($content, false)->html
        ]);
    }

    private function facebook(Item $item): string
    {
        return $this->generateArticlePart('facebook', [
            'url' => $item->url ?? ''
        ]);
    }

    private function twitter(Item $item): string
    {
        return $this->generateArticlePart('twitter', [
            'url' => $item->url ?? ''
        ]);
    }

    private function vimeo(Item $item): string
    {
        if ( ! isset($item->uri)) {
            return '';
        }

        return $this->generateArticlePart('vimeo', [
            'vimeo_id' => $vimeo_id = str_replace('im://vimeo/', '', $item->uri),
            'url' => 'https://player.vimeo.com/video/' . $vimeo_id,
            'embedId' => $item->embedId,
        ]);
    }

    private function iframely(Item $item): string
    {
        return $this->generateArticlePart('iframely', [
            'content' => $item->content ?? '',
            'links' => $item->links ?? []
        ]);
    }

    private function generateListContent(Item $item): string
    {
        if ( ! $this->isListElement($item)) {
            return '';
        }

        $listItems = str_replace('list-item', 'li', $item->getFilteredContent(['list-item']));

        switch ($item->getType()) {
            case 'x-im/unordered-list':
                return "<ul>{$listItems}</ul>";
            case 'x-im/ordered-list':
                return "<ol>{$listItems}</ol>";
        }

        return '';
    }

    private function isListElement(Item $item)
    {
        $validListTypes = ['x-im/unordered-list', 'x-im/ordered-list'];

        return in_array($item->getType(), $validListTypes, true);
    }

    private function generateArticlePart($part, $data): string
    {
        try {
            return View::generate("@base/article/{$part}", $data);
        } catch (Exception $e) {
            NewRelicLog::error('Failed to generate article body part', $e);
        }

        return '';
    }

    public function generateItem(string $type, Item $item): string
    {
        switch ($type) {
            case 'x-im/youtube':
                return $this->youtube($item);
            case 'x-im/youplay':
                return $this->youplay($item);
            case 'x-im/image':
                return $item instanceof ImageItem ? $this->image($item) : '';
            case 'x-im/link':
                return implode('', $item->toArray());
            case 'x-im/article':
                return $this->relatedArticle($item);
            case 'x-gm/review':
                return $this->review($item);
            case 'x-im/socialembed':
                return $this->socialEmbed($item);
            case 'x-im/content-part':
                return $this->contentPart($item);
            case 'x-im/htmlembed':
                return $this->htmlEmbed($item);
            case 'x-im/imagegallery':
                return $this->imageGallery($item);
            case 'x-im/table':
                return $this->table($item);
            case 'x-im/pdf':
                return $this->pdf($item);
            case 'x-im/iframely':
                return $this->iframely($item);
            default:
                return $this->element($item);

        }
    }
}
